@@ -4,7 +4,7 @@ import aiohttp
 from urllib.parse import urlparse
 from .utility import is_url, is_allowed_method, create_url_for_get_query, generate_retry_duration_list, ALLOWED_METHODS, SingleCallMethodCode, MultiCallMethodCode
 
-async def make_api_request_async(url, data_dict=None, header_dict={'Content-Type': 'application/json'}, method='POST', retries=3, duration_before_retry=[1], verbose=False):
+async def make_api_request_async(url, data_dict=None, header_dict={'Content-Type': 'application/json'}, method='POST', retries=3, duration_before_retry=[1], verbose=False, is_resp_json=True, session_timeout=300, request_timeout=300):
     if verbose:
         print("API called")
     if not is_url(url):
@@ -40,12 +40,12 @@ async def make_api_request_async(url, data_dict=None, header_dict={'Content-Type
         pass
 
     
-    async with aiohttp.ClientSession() as session:
+    async with aiohttp.ClientSession(timeout=session_timeout) as session:
     #initiate a http session. Retry it N number of times if connection not previously established
         for i in range(retries):
             try:
                 #request a session using given method, url and data_dict if any
-                async with session.request(method, url, data=data_dict, headers=header_dict) as response:
+                async with session.request(method, url, data=data_dict, headers=header_dict, timeout=request_timeout) as response:
                     if verbose:
                         print(response.status)
                     #response 
@@ -53,11 +53,18 @@ async def make_api_request_async(url, data_dict=None, header_dict={'Content-Type
                         #json_resp = await response.json()
                         if verbose:
                             print("API Finished")
-                        return {
-                            'statusCode': SingleCallMethodCode.NO_ERROR.value,
-                            'methodCode': response.status,
-                            'data': await response.json()
-                        }
+                        if is_resp_json:
+                            return {
+                                'statusCode': SingleCallMethodCode.NO_ERROR.value,
+                                'methodCode': response.status,
+                                'data': await response.json(content_type=None)
+                            }
+                        else:
+                            return {
+                                'statusCode': SingleCallMethodCode.NO_ERROR.value,
+                                'methodCode': response.status,
+                                'data': await response.text()
+                            }
                         # return await response.json()
                     else:
                         response_text = await response.text()
@@ -75,7 +82,7 @@ async def make_api_request_async(url, data_dict=None, header_dict={'Content-Type
             'methodCode': response.status
         }
     
-async def make_multiple_api_requests_async(urls_lst=[], data_dicts_lst=[], header_dicts_lst=[{'Content-Type': 'application/json'}], methods_lst=[], retries_lst=[1], duration_before_retry_lst=[[1]], verbose=False):
+async def make_multiple_api_requests_async(urls_lst=[], data_dicts_lst=[], header_dicts_lst=[{'Content-Type': 'application/json'}], methods_lst=[], retries_lst=[1], duration_before_retry_lst=[[1]], verbose=False, is_resp_json_lst=[], session_timeout=300, request_timeout=300):
     if not type(urls_lst).__name__ == 'list' or not all([is_url(url) for url in urls_lst]):
         return {
                 'statusCode': MultiCallMethodCode.ERROR_URL_PARAM.value
@@ -108,19 +115,19 @@ async def make_multiple_api_requests_async(urls_lst=[], data_dicts_lst=[], heade
     
     api_job_lsts = []
     for idx, url in enumerate(urls_lst):
-        api_job_lsts.append(make_api_request_async(url=url, method=methods_lst[idx], retries=retries_lst[idx], duration_before_retry=duration_before_retry_lst[idx], data_dict=data_dicts_lst[idx], header_dict=header_dicts_lst[idx]), verbose=verbose)
+        api_job_lsts.append(make_api_request_async(url=url, method=methods_lst[idx], retries=retries_lst[idx], duration_before_retry=duration_before_retry_lst[idx], data_dict=data_dicts_lst[idx], header_dict=header_dicts_lst[idx], verbose=verbose, is_resp_json=is_resp_json_lst[idx], session_timeout=session_timeout, request_timeout=request_timeout))
     api_responses = await asyncio.gather(*api_job_lsts)
     return api_responses
 
-def create_task_api_request_call(url, data_dict=None, header_dict={'Content-Type': 'application/json'}, method='POST', retries=3, duration_before_retry=[1,2,3], verbose=False):
-    return asyncio.create_task(make_api_request_async(url=url, method=method, retries=retries, duration_before_retry=duration_before_retry, data_dict=data_dict, header_dict=header_dict, verbose=verbose))
+def create_task_api_request_call(url, data_dict=None, header_dict={'Content-Type': 'application/json'}, method='POST', retries=3, duration_before_retry=[1,2,3], verbose=False, session_timeout=300, request_timeout=300):
+    return asyncio.create_task(make_api_request_async(url=url, method=method, retries=retries, duration_before_retry=duration_before_retry, data_dict=data_dict, header_dict=header_dict, verbose=verbose, session_timeout=session_timeout, request_timeout=request_timeout))
 
-def create_task_mult_api_request_call(urls_lst=[], data_dicts_lst=[], header_dicts_lst=[{'Content-Type': 'application/json'}], methods_lst=[], retries_lst=[1], duration_before_retry_lst=[[1]], verbose=False):
-    return asyncio.create_task(make_multiple_api_requests_async(urls_lst=urls_lst, data_dicts_lst=data_dicts_lst, header_dicts_lst=header_dicts_lst, methods_lst=methods_lst, retries_lst=retries_lst, duration_before_retry_lst=duration_before_retry_lst, verbose=verbose))
+def create_task_mult_api_request_call(urls_lst=[], data_dicts_lst=[], header_dicts_lst=[{'Content-Type': 'application/json'}], methods_lst=[], retries_lst=[1], duration_before_retry_lst=[[1]], verbose=False, session_timeout=300, request_timeout=300):
+    return asyncio.create_task(make_multiple_api_requests_async(urls_lst=urls_lst, data_dicts_lst=data_dicts_lst, header_dicts_lst=header_dicts_lst, methods_lst=methods_lst, retries_lst=retries_lst, duration_before_retry_lst=duration_before_retry_lst, verbose=verbose, is_resp_json_lst=[], session_timeout=session_timeout, request_timeout=request_timeout))
 
 
 # PRE-BUILT METHODS DEVELOPED FOR CUSTOM NEED
-async def make_post_api_request_async(url, data_dict=None, header_dict={'Content-Type': 'application/json'}, method='POST', retries=3, duration_before_retry=[1,2,3], verbose=False):
+async def make_post_api_request_async(url, data_dict=None, header_dict={'Content-Type': 'application/json'}, method='POST', retries=3, duration_before_retry=[1,2,3], verbose=False, is_resp_json=True, session_timeout=300, request_timeout=300):
     if verbose:
         print("API called")
     if not is_url(url):
@@ -156,12 +163,12 @@ async def make_post_api_request_async(url, data_dict=None, header_dict={'Content
         pass
 
     
-    async with aiohttp.ClientSession() as session:
+    async with aiohttp.ClientSession(timeout=session_timeout) as session:
     #initiate a http session. Retry it N number of times if connection not previously established
         for i in range(retries):
             try:
                 #request a session using given method, url and data_dict if any
-                async with session.request(method, url, data=data_dict, headers=header_dict) as response:
+                async with session.request(method, url, data=data_dict, headers=header_dict, timeout=request_timeout) as response:
                     if verbose:
                         print(response.status)
                     #response 
@@ -169,11 +176,18 @@ async def make_post_api_request_async(url, data_dict=None, header_dict={'Content
                         #json_resp = await response.json()
                         if verbose:
                             print("API Finished")
-                        return {
-                            'statusCode': SingleCallMethodCode.NO_ERROR.value,
-                            'methodCode': response.status,
-                            'data': await response.json()
-                        }
+                        if is_resp_json:
+                            return {
+                                'statusCode': SingleCallMethodCode.NO_ERROR.value,
+                                'methodCode': response.status,
+                                'data': await response.json(content_type=None)
+                            }
+                        else:
+                            return {
+                                'statusCode': SingleCallMethodCode.NO_ERROR.value,
+                                'methodCode': response.status,
+                                'data': await response.text()
+                            }
                         # return await response.json()
                     else:
                         response_text = await response.text()
@@ -191,5 +205,5 @@ async def make_post_api_request_async(url, data_dict=None, header_dict={'Content
             'methodCode': response.status
         }
     
-def create_task_post_api_request_call(url, data_dict=None, header_dict={'Content-Type': 'application/json'}, retries=3, duration_before_retry=[1,2,3], verbose=False):
+def create_task_post_api_request_call(url, data_dict=None, header_dict={'Content-Type': 'application/json'}, retries=3, duration_before_retry=[1,2,3], verbose=False, is_resp_json=True):
     return asyncio.create_task(make_post_api_request_async(url=url, retries=retries, duration_before_retry=duration_before_retry, data_dict=data_dict, header_dict=header_dict, verbose=verbose))
